@@ -19,7 +19,7 @@
 from __future__ import (absolute_import, division, print_function)
 __metaclass__ = type
 
-from collections import Sequence
+import re
 
 from ansible.errors.yaml_strings import (
     YAML_COMMON_DICT_ERROR,
@@ -29,8 +29,10 @@ from ansible.errors.yaml_strings import (
     YAML_COMMON_UNQUOTED_COLON_ERROR,
     YAML_COMMON_UNQUOTED_VARIABLE_ERROR,
     YAML_POSITION_DETAILS,
+    YAML_AND_SHORTHAND_ERROR,
 )
 from ansible.module_utils._text import to_native, to_text
+from ansible.module_utils.common._collections_compat import Sequence
 
 
 class AnsibleError(Exception):
@@ -116,9 +118,18 @@ class AnsibleError(Exception):
                 prev_line = to_text(prev_line)
                 if target_line:
                     stripped_line = target_line.replace(" ", "")
-                    arrow_line = (" " * (col_number - 1)) + "^ here"
-                    # header_line = ("=" * 73)
-                    error_message += "\nThe offending line appears to be:\n\n%s\n%s\n%s\n" % (prev_line.rstrip(), target_line.rstrip(), arrow_line)
+
+                    # Check for k=v syntax in addition to YAML syntax and set the appropriate error position,
+                    # arrow index
+                    if re.search(r'\w+(\s+)?=(\s+)?[\w/-]+', prev_line):
+                        error_position = prev_line.rstrip().find('=')
+                        arrow_line = (" " * error_position) + "^ here"
+                        error_message = YAML_POSITION_DETAILS % (src_file, line_number - 1, error_position + 1)
+                        error_message += "\nThe offending line appears to be:\n\n%s\n%s\n\n" % (prev_line.rstrip(), arrow_line)
+                        error_message += YAML_AND_SHORTHAND_ERROR
+                    else:
+                        arrow_line = (" " * (col_number - 1)) + "^ here"
+                        error_message += "\nThe offending line appears to be:\n\n%s\n%s\n%s\n" % (prev_line.rstrip(), target_line.rstrip(), arrow_line)
 
                     # TODO: There may be cases where there is a valid tab in a line that has other errors.
                     if '\t' in target_line:
@@ -139,6 +150,9 @@ class AnsibleError(Exception):
                         error_message += YAML_COMMON_UNQUOTED_COLON_ERROR
                     # otherwise, check for some common quoting mistakes
                     else:
+                        # FIXME: This needs to split on the first ':' to account for modules like lineinfile
+                        # that may have lines that contain legitimate colons, e.g., line: 'i ALL= (ALL) NOPASSWD: ALL'
+                        # and throw off the quote matching logic.
                         parts = target_line.split(":")
                         if len(parts) > 1:
                             middle = parts[1].strip()
@@ -210,22 +224,27 @@ class AnsibleAuthenticationFailure(AnsibleConnectionFailure):
     pass
 
 
-class AnsibleFilterError(AnsibleRuntimeError):
-    ''' a templating failure '''
-    pass
-
-
-class AnsibleLookupError(AnsibleRuntimeError):
-    ''' a lookup failure '''
-    pass
-
-
 class AnsibleCallbackError(AnsibleRuntimeError):
     ''' a callback failure '''
     pass
 
 
-class AnsibleUndefinedVariable(AnsibleRuntimeError):
+class AnsibleTemplateError(AnsibleRuntimeError):
+    '''A template related errror'''
+    pass
+
+
+class AnsibleFilterError(AnsibleTemplateError):
+    ''' a templating failure '''
+    pass
+
+
+class AnsibleLookupError(AnsibleTemplateError):
+    ''' a lookup failure '''
+    pass
+
+
+class AnsibleUndefinedVariable(AnsibleTemplateError):
     ''' a templating failure '''
     pass
 
@@ -238,10 +257,12 @@ class AnsibleFileNotFound(AnsibleRuntimeError):
         self.file_name = file_name
         self.paths = paths
 
+        if message:
+            message += "\n"
         if self.file_name:
-            if message:
-                message += "\n"
             message += "Could not find or access '%s'" % to_text(self.file_name)
+        else:
+            message += "Could not find file"
 
         if self.paths and isinstance(self.paths, Sequence):
             searched = to_text('\n\t'.join(self.paths))
@@ -291,3 +312,8 @@ class AnsibleActionFail(AnsibleAction):
 class _AnsibleActionDone(AnsibleAction):
     ''' an action runtime early exit'''
     pass
+
+
+class AnsibleFilterTypeError(AnsibleTemplateError, TypeError):
+    ''' a Jinja filter templating failure due to bad type'''
+    pass
