@@ -81,6 +81,8 @@ def build_pyxcli_command(fields):
     """ Builds the args for pyxcli using the exact args from ansible"""
     pyxcli_args = {}
     for field in fields:
+        if not fields[field]:
+            continue
         if field in AVAILABLE_PYXCLI_FIELDS and fields[field] != '':
             pyxcli_args[field] = fields[field]
     return pyxcli_args
