@@ -2,21 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 # (c) 2015, René Moser <mail@renemoser.net>
-#
-# This file is part of Ansible
-#
-# Ansible is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Ansible is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Ansible. If not, see <http://www.gnu.org/licenses/>.
+# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
 ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['stableinterface'],
@@ -30,65 +16,93 @@ short_description: Manages accounts on Apache CloudStack based clouds.
 description:
     - Create, disable, lock, enable and remove accounts.
 version_added: '2.0'
-author: "René Moser (@resmo)"
+author: René Moser (@resmo)
 options:
   name:
     description:
       - Name of account.
+    type: str
     required: true
   username:
     description:
       - Username of the user to be created if account did not exist.
-      - Required on C(state=present).
+      - Required on I(state=present).
+    type: str
   password:
     description:
       - Password of the user to be created if account did not exist.
-      - Required on C(state=present).
+      - Required on I(state=present) if I(ldap_domain) is not set.
+    type: str
   first_name:
     description:
       - First name of the user to be created if account did not exist.
-      - Required on C(state=present).
+      - Required on I(state=present) if I(ldap_domain) is not set.
+    type: str
   last_name:
     description:
       - Last name of the user to be created if account did not exist.
-      - Required on C(state=present).
+      - Required on I(state=present) if I(ldap_domain) is not set.
+    type: str
   email:
     description:
       - Email of the user to be created if account did not exist.
-      - Required on C(state=present).
+      - Required on I(state=present) if I(ldap_domain) is not set.
+    type: str
   timezone:
     description:
       - Timezone of the user to be created if account did not exist.
+    type: str
   network_domain:
     description:
       - Network domain of the account.
+    type: str
   account_type:
     description:
       - Type of the account.
-    default: 'user'
-    choices: [ 'user', 'root_admin', 'domain_admin' ]
+    type: str
+    choices: [ user, root_admin, domain_admin ]
+    default: user
   domain:
     description:
       - Domain the account is related to.
-    default: 'ROOT'
+    type: str
+    default: ROOT
+  role:
+    description:
+      - Creates the account under the specified role name or id.
+    type: str
+    version_added: '2.8'
+  ldap_domain:
+    description:
+      - Name of the LDAP group or OU to bind.
+      - If set, account will be linked to LDAP.
+    type: str
+    version_added: '2.8'
+  ldap_type:
+    description:
+      - Type of the ldap name. GROUP or OU, defaults to GROUP.
+    type: str
+    choices: [ GROUP, OU ]
+    default: GROUP
+    version_added: '2.8'
   state:
     description:
       - State of the account.
       - C(unlocked) is an alias for C(enabled).
-    default: 'present'
-    choices: [ 'present', 'absent', 'enabled', 'disabled', 'locked', 'unlocked' ]
+    type: str
+    choices: [ present, absent, enabled, disabled, locked, unlocked ]
+    default: present
   poll_async:
     description:
       - Poll async jobs until job has finished.
     type: bool
-    default: 'yes'
+    default: yes
 extends_documentation_fragment: cloudstack
 '''
 
 EXAMPLES = '''
-# create an account in domain 'CUSTOMERS'
-- local_action:
-    module: cs_account
+- name: create an account in domain 'CUSTOMERS'
+  cs_account:
     name: customer_xy
     username: customer_xy
     password: S3Cur3
@@ -96,34 +110,52 @@ EXAMPLES = '''
     first_name: John
     email: john.doe@example.com
     domain: CUSTOMERS
+    role: Domain Admin
+  delegate_to: localhost
 
-# Lock an existing account in domain 'CUSTOMERS'
-- local_action:
-    module: cs_account
+- name: Lock an existing account in domain 'CUSTOMERS'
+  cs_account:
     name: customer_xy
     domain: CUSTOMERS
     state: locked
+  delegate_to: localhost
 
-# Disable an existing account in domain 'CUSTOMERS'
-- local_action:
-    module: cs_account
+- name: Disable an existing account in domain 'CUSTOMERS'
+  cs_account:
     name: customer_xy
     domain: CUSTOMERS
     state: disabled
+  delegate_to: localhost
 
-# Enable an existing account in domain 'CUSTOMERS'
-- local_action:
-    module: cs_account
+- name: Enable an existing account in domain 'CUSTOMERS'
+  cs_account:
     name: customer_xy
     domain: CUSTOMERS
     state: enabled
+  delegate_to: localhost
 
-# Remove an account in domain 'CUSTOMERS'
-- local_action:
-    module: cs_account
+- name: Remove an account in domain 'CUSTOMERS'
+  cs_account:
     name: customer_xy
     domain: CUSTOMERS
     state: absent
+  delegate_to: localhost
+
+- name: Create a single user LDAP account in domain 'CUSTOMERS'
+  cs_account:
+    name: customer_xy
+    username: customer_xy
+    domain: CUSTOMERS
+    ldap_domain: cn=customer_xy,cn=team_xy,ou=People,dc=domain,dc=local
+  delegate_to: localhost
+
+- name: Create a LDAP account in domain 'CUSTOMERS' and bind it to a LDAP group
+  cs_account:
+    name: team_xy
+    username: customer_xy
+    domain: CUSTOMERS
+    ldap_domain: cn=team_xy,ou=People,dc=domain,dc=local
+  delegate_to: localhost
 '''
 
 RETURN = '''
@@ -131,33 +163,38 @@ RETURN = '''
 id:
   description: UUID of the account.
   returned: success
-  type: string
+  type: str
   sample: 87b1e0ce-4e01-11e4-bb66-0050569e64b8
 name:
   description: Name of the account.
   returned: success
-  type: string
+  type: str
   sample: linus@example.com
 account_type:
   description: Type of the account.
   returned: success
-  type: string
+  type: str
   sample: user
 state:
   description: State of the account.
   returned: success
-  type: string
+  type: str
   sample: enabled
 network_domain:
   description: Network domain of the account.
   returned: success
-  type: string
+  type: str
   sample: example.local
 domain:
   description: Domain the account is related.
   returned: success
-  type: string
+  type: str
   sample: ROOT
+role:
+  description: The role name of the account
+  returned: success
+  type: str
+  sample: Domain Admin
 '''
 
 # import cloudstack common
@@ -175,6 +212,7 @@ class AnsibleCloudStackAccount(AnsibleCloudStack):
         super(AnsibleCloudStackAccount, self).__init__(module)
         self.returns = {
             'networkdomain': 'network_domain',
+            'rolename': 'role',
         }
         self.account = None
         self.account_types = {
@@ -183,6 +221,21 @@ class AnsibleCloudStackAccount(AnsibleCloudStack):
             'domain_admin': 2,
         }
 
+    def get_role_id(self):
+        role_param = self.module.params.get('role')
+        role_id = None
+
+        if role_param:
+            role_list = self.query_api('listRoles')
+            for role in role_list['role']:
+                if role_param in [role['name'], role['id']]:
+                    role_id = role['id']
+
+            if not role_id:
+                self.module.fail_json(msg="Role not found: %s" % role_param)
+
+        return role_id
+
     def get_account_type(self):
         account_type = self.module.params.get('account_type')
         return self.account_types[account_type]
@@ -254,35 +307,78 @@ class AnsibleCloudStackAccount(AnsibleCloudStack):
         return account
 
     def present_account(self):
-        required_params = [
-            'email',
-            'username',
-            'password',
-            'first_name',
-            'last_name',
-        ]
-        self.module.fail_on_missing_params(required_params=required_params)
-
         account = self.get_account()
 
         if not account:
             self.result['changed'] = True
 
+            if self.module.params.get('ldap_domain'):
+                required_params = [
+                    'domain',
+                    'username',
+                ]
+                self.module.fail_on_missing_params(required_params=required_params)
+
+                account = self.create_ldap_account(account)
+
+            else:
+                required_params = [
+                    'email',
+                    'username',
+                    'password',
+                    'first_name',
+                    'last_name',
+                ]
+                self.module.fail_on_missing_params(required_params=required_params)
+
+                account = self.create_account(account)
+
+        return account
+
+    def create_ldap_account(self, account):
+        args = {
+            'account': self.module.params.get('name'),
+            'domainid': self.get_domain(key='id'),
+            'accounttype': self.get_account_type(),
+            'networkdomain': self.module.params.get('network_domain'),
+            'username': self.module.params.get('username'),
+            'timezone': self.module.params.get('timezone'),
+            'roleid': self.get_role_id()
+        }
+        if not self.module.check_mode:
+            res = self.query_api('ldapCreateAccount', **args)
+            account = res['account']
+
             args = {
                 'account': self.module.params.get('name'),
                 'domainid': self.get_domain(key='id'),
                 'accounttype': self.get_account_type(),
-                'networkdomain': self.module.params.get('network_domain'),
-                'username': self.module.params.get('username'),
-                'password': self.module.params.get('password'),
-                'firstname': self.module.params.get('first_name'),
-                'lastname': self.module.params.get('last_name'),
-                'email': self.module.params.get('email'),
-                'timezone': self.module.params.get('timezone')
+                'ldapdomain': self.module.params.get('ldap_domain'),
+                'type': self.module.params.get('ldap_type')
             }
-            if not self.module.check_mode:
-                res = self.query_api('createAccount', **args)
-                account = res['account']
+
+            self.query_api('linkAccountToLdap', **args)
+
+        return account
+
+    def create_account(self, account):
+        args = {
+            'account': self.module.params.get('name'),
+            'domainid': self.get_domain(key='id'),
+            'accounttype': self.get_account_type(),
+            'networkdomain': self.module.params.get('network_domain'),
+            'username': self.module.params.get('username'),
+            'password': self.module.params.get('password'),
+            'firstname': self.module.params.get('first_name'),
+            'lastname': self.module.params.get('last_name'),
+            'email': self.module.params.get('email'),
+            'timezone': self.module.params.get('timezone'),
+            'roleid': self.get_role_id()
+        }
+        if not self.module.check_mode:
+            res = self.query_api('createAccount', **args)
+            account = res['account']
+
         return account
 
     def absent_account(self):
@@ -323,6 +419,9 @@ def main():
         username=dict(),
         password=dict(no_log=True),
         timezone=dict(),
+        role=dict(),
+        ldap_domain=dict(),
+        ldap_type=dict(choices=['GROUP', 'OU'], default='GROUP'),
         poll_async=dict(type='bool', default=True),
     ))
 
