@@ -33,55 +33,43 @@ options:
     description:
       - the password to authenticate with
       - you can use PROXMOX_PASSWORD environment variable
-    default: null
-    required: false
   validate_certs:
     description:
       - enable / disable https certificate verification
-    default: false
-    required: false
+    default: 'no'
     type: bool
   node:
     description:
       - Proxmox VE node, when you will operate with template
-    default: null
     required: true
   src:
     description:
       - path to uploaded file
       - required only for C(state=present)
-    default: null
-    required: false
     aliases: ['path']
   template:
     description:
       - the template name
       - required only for states C(absent), C(info)
-    default: null
-    required: false
   content_type:
     description:
       - content type
       - required only for C(state=present)
     default: 'vztmpl'
-    required: false
     choices: ['vztmpl', 'iso']
   storage:
     description:
       - target storage
     default: 'local'
-    required: false
   timeout:
     description:
       - timeout for operations
     default: 30
-    required: false
   force:
     description:
       - can be used only with C(state=present), exists template will be overwritten
-    default: false
-    required: false
     type: bool
+    default: 'no'
   state:
     description:
      - Indicate desired state of the template
@@ -90,7 +78,7 @@ options:
 notes:
   - Requires proxmoxer and requests modules on host. This modules can be installed with pip.
 requirements: [ "proxmoxer", "requests" ]
-author: "Sergei Antipov @UnderGreen"
+author: Sergei Antipov (@UnderGreen)
 '''
 
 EXAMPLES = '''
@@ -148,7 +136,7 @@ def get_template(proxmox, node, storage, content_type, template):
 
 
 def upload_template(module, proxmox, api_host, node, storage, content_type, realpath, timeout):
-    taskid = proxmox.nodes(node).storage(storage).upload.post(content=content_type, filename=open(realpath))
+    taskid = proxmox.nodes(node).storage(storage).upload.post(content=content_type, filename=open(realpath, 'rb'))
     while timeout:
         task_status = proxmox.nodes(api_host.split('.')[0]).tasks(taskid).status.get()
         if task_status['status'] == 'stopped' and task_status['exitstatus'] == 'OK':
