@@ -2,23 +2,12 @@
 # -*- coding: utf-8 -*-
 
 # Copyright 2014 Benjamin Curtis <benjamin.curtis@gmail.com>
-#
-# This file is part of Ansible
-#
-# Ansible is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Ansible is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
+# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-ANSIBLE_METADATA = {'metadata_version': '1.0',
+from __future__ import absolute_import, division, print_function
+__metaclass__ = type
+
+ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['preview'],
                     'supported_by': 'community'}
 
@@ -43,32 +32,23 @@ options:
   user:
     description:
       - The username of the person doing the deployment
-    required: false
-    default: None
   repo:
     description:
       - URL of the project repository
-    required: false
-    default: None
   revision:
     description:
       - A hash, number, tag, or other identifier showing what revision was deployed
-    required: false
-    default: None
   url:
     description:
       - Optional URL to submit the notification to.
-    required: false
     default: "https://api.honeybadger.io/v1/deploys"
   validate_certs:
     description:
       - If C(no), SSL certificates for the target url will not be validated. This should only be used
         on personally controlled sites using self-signed certificates.
-    required: false
+    type: bool
     default: 'yes'
-    choices: ['yes', 'no']
 
-requirements: []
 '''
 
 EXAMPLES = '''
@@ -82,12 +62,13 @@ EXAMPLES = '''
 
 RETURN = '''# '''
 
-import urllib
+import traceback
+
+from ansible.module_utils.basic import AnsibleModule
+from ansible.module_utils.six.moves.urllib.parse import urlencode
+from ansible.module_utils._text import to_native
+from ansible.module_utils.urls import fetch_url
 
-# import module snippets
-from ansible.module_utils.basic import *
-from ansible.module_utils.pycompat24 import get_exception
-from ansible.module_utils.urls import *
 
 # ===========================================
 # Module execution.
@@ -131,17 +112,16 @@ def main():
         module.exit_json(changed=True)
 
     try:
-        data = urllib.urlencode(params)
+        data = urlencode(params)
         response, info = fetch_url(module, url, data=data)
-    except Exception:
-        e = get_exception()
-        module.fail_json(msg='Unable to notify Honeybadger: %s' % e)
+    except Exception as e:
+        module.fail_json(msg='Unable to notify Honeybadger: %s' % to_native(e), exception=traceback.format_exc())
     else:
         if info['status'] == 201:
             module.exit_json(changed=True)
         else:
             module.fail_json(msg="HTTP result code: %d connecting to %s" % (info['status'], url))
 
+
 if __name__ == '__main__':
     main()
-
