@@ -1,10 +1,11 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-# Ansible module to manage Big Monitoring Fabric service chains
-# (c) 2016, Ted Elhourani <ted@bigswitch.com>
+# Copyright: (c) 2016, Ted Elhourani <ted@bigswitch.com>
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
+# Ansible module to manage Big Monitoring Fabric service chains
+
 from __future__ import absolute_import, division, print_function
 __metaclass__ = type
 
@@ -67,7 +68,7 @@ options:
        on personally controlled devices using self-signed certificates.
     required: false
     default: true
-    choices: [true, false]
+    type: bool
   access_token:
     description:
      - Bigmon access token. If this isn't set, the environment variable C(BIGSWITCH_ACCESS_TOKEN) is used.
@@ -92,7 +93,7 @@ import os
 import traceback
 
 from ansible.module_utils.basic import AnsibleModule
-from ansible.module_utils.bigswitch_utils import Rest
+from ansible.module_utils.network.bigswitch.bigswitch import Rest
 from ansible.module_utils._text import to_native
 
 
@@ -113,25 +114,25 @@ def policy(module):
     controller = module.params['controller']
 
     rest = Rest(module,
-                {'content-type': 'application/json', 'Cookie': 'session_cookie='+access_token},
-                'https://'+controller+':8443/api/v1/data/controller/applications/bigtap')
+                {'content-type': 'application/json', 'Cookie': 'session_cookie=' + access_token},
+                'https://' + controller + ':8443/api/v1/data/controller/applications/bigtap')
 
     if name is None:
         module.fail_json(msg='parameter `name` is missing')
 
     response = rest.get('policy?config=true', data={})
     if response.status_code != 200:
-        module.fail_json(msg="failed to obtain existing policy config: {}".format(response.json['description']))
+        module.fail_json(msg="failed to obtain existing policy config: {0}".format(response.json['description']))
 
     config_present = False
 
     matching = [policy for policy in response.json
                 if policy['name'] == name and
-                   policy['duration'] == duration and
-                   policy['delivery-packet-count'] == delivery_packet_count and
-                   policy['policy-description'] == policy_description and
-                   policy['action'] == action and
-                   policy['priority'] == priority]
+                policy['duration'] == duration and
+                policy['delivery-packet-count'] == delivery_packet_count and
+                policy['policy-description'] == policy_description and
+                policy['action'] == action and
+                policy['priority'] == priority]
 
     if matching:
         config_present = True
@@ -143,22 +144,23 @@ def policy(module):
         module.exit_json(changed=False)
 
     if state in ('present'):
-        data={'name': name, 'action': action, 'policy-description': policy_description,
-              'priority': priority, 'duration': duration, 'start-time': start_time,
-              'delivery-packet-count': delivery_packet_count }
+        data = {'name': name, 'action': action, 'policy-description': policy_description,
+                'priority': priority, 'duration': duration, 'start-time': start_time,
+                'delivery-packet-count': delivery_packet_count}
 
         response = rest.put('policy[name="%s"]' % name, data=data)
         if response.status_code == 204:
             module.exit_json(changed=True)
         else:
-            module.fail_json(msg="error creating policy '{}': {}".format(name, response.json['description']))
+            module.fail_json(msg="error creating policy '{0}': {1}".format(name, response.json['description']))
 
     if state in ('absent'):
         response = rest.delete('policy[name="%s"]' % name, data={})
         if response.status_code == 204:
             module.exit_json(changed=True)
         else:
-            module.fail_json(msg="error deleting policy '{}': {}".format(name, response.json['description']))
+            module.fail_json(msg="error deleting policy '{0}': {1}".format(name, response.json['description']))
+
 
 def main():
     module = AnsibleModule(
@@ -168,7 +170,7 @@ def main():
             action=dict(choices=['forward', 'drop', 'capture', 'flow-gen'], default='forward'),
             priority=dict(type='int', default=100),
             duration=dict(type='int', default=0),
-            start_time=dict(type='str', default=datetime.datetime.now().isoformat()+'+00:00'),
+            start_time=dict(type='str', default=datetime.datetime.now().isoformat() + '+00:00'),
             delivery_packet_count=dict(type='int', default=0),
             controller=dict(type='str', required=True),
             state=dict(choices=['present', 'absent'], default='present'),
@@ -182,5 +184,6 @@ def main():
     except Exception as e:
         module.fail_json(msg=to_native(e), exception=traceback.format_exc())
 
+
 if __name__ == '__main__':
     main()
