@@ -28,6 +28,10 @@ short_description: Manages BFD global configuration on HUAWEI CloudEngine device
 description:
     - Manages BFD global configuration on HUAWEI CloudEngine devices.
 author: QijunPan (@QijunPan)
+notes:
+  - This module requires the netconf system service be enabled on the remote device being managed.
+  - Recommended connection is C(netconf).
+  - This module also works with C(local) connections for legacy playbooks.
 options:
     bfd_enable:
         description:
