@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2017 F5 Networks Inc.
+# Copyright: (c) 2017, F5 Networks Inc.
 # GNU General Public License v3.0 (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
 from __future__ import absolute_import, division, print_function
@@ -25,6 +25,7 @@ options:
   name:
     description:
       - The name of the vCMP guest to manage.
+    type: str
     required: True
   vlans:
     description:
@@ -34,11 +35,19 @@ options:
       - The order of these VLANs is not important; in fact, it's ignored. This module will
         order the VLANs for you automatically. Therefore, if you deliberately re-order them
         in subsequent tasks, you will find that this module will B(not) register a change.
+    type: list
   initial_image:
     description:
       - Specifies the base software release ISO image file for installing the TMOS
         hypervisor instance and any licensed BIG-IP modules onto the guest's virtual
         disk. When creating a new guest, this parameter is required.
+    type: str
+  initial_hotfix:
+    description:
+      - Specifies the hotfix ISO image file which will be applied on top of the base
+        image.
+    type: str
+    version_added: 2.9
   mgmt_network:
     description:
       - Specifies the method by which the management address is used in the vCMP guest.
@@ -60,6 +69,7 @@ options:
         management network. This immediately connects all of the guest's VMs to the
         physical management network. Changing this property while the guest is in the
         C(configured) or C(provisioned) state has no immediate effect.
+    type: str
     choices:
       - bridged
       - isolated
@@ -77,11 +87,13 @@ options:
         parameter is required if the C(mgmt_network) parameter is C(bridged).
       - When creating a new guest, if you do not specify a network or network mask,
         a default of C(/24) (C(255.255.255.0)) will be assumed.
+    type: str
   mgmt_route:
     description:
       - Specifies the gateway address for the C(mgmt_address).
       - If this value is not specified when creating a new guest, it is set to C(none).
       - The value C(none) can be used during an update to remove this value.
+    type: str
   state:
     description:
       - The state of the vCMP guest on the system. Each state implies the actions of
@@ -97,13 +109,14 @@ options:
       - When C(present), ensures the guest is properly provisioned and starts
         the guest so that it is in a running state.
       - When C(absent), removes the vCMP from the system.
-    default: "present"
+    type: str
     choices:
       - configured
       - disabled
       - provisioned
       - present
       - absent
+    default: present
   cores_per_slot:
     description:
       - Specifies the number of cores that the system allocates to the guest.
@@ -113,9 +126,11 @@ options:
       - The number you can specify depends on the type of hardware you have.
       - In the event of a reboot, the system persists the guest to the same slot on
         which it ran prior to the reboot.
+    type: int
   partition:
     description:
       - Device partition to manage resources on.
+    type: str
     default: Common
   number_of_slots:
     description:
@@ -124,6 +139,7 @@ options:
         it is assigned to.
       - Possible values are dependent on the type of blades being used in this cluster.
       - The default value depends on the type of blades being used in this cluster.
+    type: int
     version_added: 2.7
   min_number_of_slots:
     description:
@@ -133,6 +149,7 @@ options:
       - If at the end of any allocation attempt the guest is not assigned to at least
         this many slots, the attempt fails and the change that initiated it is reverted.
       - A guest's C(min_number_of_slots) value cannot be greater than its C(number_of_slots).
+    type: int
     version_added: 2.7
   allowed_slots:
     description:
@@ -144,6 +161,7 @@ options:
         never assigned to the same slot.
       - By default this list includes every available slot in the cluster. This means,
         by default, the guest may be assigned to any slot.
+    type: list
     version_added: 2.7
 notes:
   - This module can take a lot of time to deploy vCMP guests. This is an intrinsic
@@ -157,32 +175,33 @@ notes:
 extends_documentation_fragment: f5
 author:
   - Tim Rupp (@caphrim007)
+  - Wojciech Wypior (@wojtek0806)
 '''
 
 EXAMPLES = r'''
 - name: Create a vCMP guest
   bigip_vcmp_guest:
     name: foo
-    password: secret
-    server: lb.mydomain.com
-    state: present
-    user: admin
     mgmt_network: bridge
     mgmt_address: 10.20.30.40/24
+    provider:
+      password: secret
+      server: lb.mydomain.com
+      user: admin
   delegate_to: localhost
 
 - name: Create a vCMP guest with specific VLANs
   bigip_vcmp_guest:
     name: foo
-    password: secret
-    server: lb.mydomain.com
-    state: present
-    user: admin
     mgmt_network: bridge
     mgmt_address: 10.20.30.40/24
     vlans:
       - vlan1
       - vlan2
+    provider:
+      password: secret
+      server: lb.mydomain.com
+      user: admin
   delegate_to: localhost
 
 - name: Remove vCMP guest and disk
@@ -190,6 +209,10 @@ EXAMPLES = r'''
     name: guest1
     state: absent
     delete_virtual_disk: yes
+    provider:
+      password: secret
+      server: lb.mydomain.com
+      user: admin
   register: result
 '''
 
@@ -208,35 +231,23 @@ from ansible.module_utils.basic import env_fallback
 from collections import namedtuple
 
 try:
-    from library.module_utils.network.f5.bigip import HAS_F5SDK
-    from library.module_utils.network.f5.bigip import F5Client
+    from library.module_utils.network.f5.bigip import F5RestClient
     from library.module_utils.network.f5.common import F5ModuleError
     from library.module_utils.network.f5.common import AnsibleF5Parameters
-    from library.module_utils.network.f5.common import cleanup_tokens
     from library.module_utils.network.f5.common import fq_name
     from library.module_utils.network.f5.common import f5_argument_spec
+    from library.module_utils.network.f5.urls import parseStats
     from library.module_utils.network.f5.ipaddress import is_valid_ip
     from library.module_utils.compat.ipaddress import ip_interface
-    try:
-        from library.module_utils.network.f5.common import iControlUnexpectedHTTPError
-        from f5.utils.responses.handlers import Stats
-    except ImportError:
-        HAS_F5SDK = False
 except ImportError:
-    from ansible.module_utils.network.f5.bigip import HAS_F5SDK
-    from ansible.module_utils.network.f5.bigip import F5Client
+    from ansible.module_utils.network.f5.bigip import F5RestClient
     from ansible.module_utils.network.f5.common import F5ModuleError
     from ansible.module_utils.network.f5.common import AnsibleF5Parameters
-    from ansible.module_utils.network.f5.common import cleanup_tokens
     from ansible.module_utils.network.f5.common import fq_name
     from ansible.module_utils.network.f5.common import f5_argument_spec
+    from ansible.module_utils.network.f5.urls import parseStats
     from ansible.module_utils.network.f5.ipaddress import is_valid_ip
     from ansible.module_utils.compat.ipaddress import ip_interface
-    try:
-        from ansible.module_utils.network.f5.common import iControlUnexpectedHTTPError
-        from f5.utils.responses.handlers import Stats
-    except ImportError:
-        HAS_F5SDK = False
 
 
 class Parameters(AnsibleF5Parameters):
@@ -245,6 +256,7 @@ class Parameters(AnsibleF5Parameters):
         'managementNetwork': 'mgmt_network',
         'managementIp': 'mgmt_address',
         'initialImage': 'initial_image',
+        'initialHotfix': 'initial_hotfix',
         'virtualDisk': 'virtual_disk',
         'coresPerSlot': 'cores_per_slot',
         'slots': 'number_of_slots',
@@ -257,6 +269,7 @@ class Parameters(AnsibleF5Parameters):
         'managementNetwork',
         'managementIp',
         'initialImage',
+        'initialHotfix',
         'managementGw',
         'state',
         'coresPerSlot',
@@ -270,6 +283,7 @@ class Parameters(AnsibleF5Parameters):
         'mgmt_network',
         'mgmt_address',
         'initial_image',
+        'initial_hotfix',
         'mgmt_route',
         'name',
         'cores_per_slot',
@@ -283,6 +297,7 @@ class Parameters(AnsibleF5Parameters):
         'mgmt_network',
         'mgmt_address',
         'initial_image',
+        'initial_hotfix',
         'mgmt_route',
         'state',
         'cores_per_slot',
@@ -291,16 +306,12 @@ class Parameters(AnsibleF5Parameters):
         'allowed_slots',
     ]
 
-    def to_return(self):
-        result = {}
-        try:
-            for returnable in self.returnables:
-                result[returnable] = getattr(self, returnable)
-            result = self._filter_params(result)
-        except Exception:
-            pass
-        return result
 
+class ApiParameters(Parameters):
+    pass
+
+
+class ModuleParameters(Parameters):
     @property
     def mgmt_route(self):
         if self._values['mgmt_route'] is None:
@@ -311,7 +322,7 @@ class Parameters(AnsibleF5Parameters):
             return self._values['mgmt_route']
         else:
             raise F5ModuleError(
-                "The specified 'mgmt_route' is not a valid IP address"
+                "The specified 'mgmt_route' is not a valid IP address."
             )
 
     @property
@@ -323,12 +334,11 @@ class Parameters(AnsibleF5Parameters):
             return str(addr.with_prefixlen)
         except ValueError:
             raise F5ModuleError(
-                "The specified 'mgmt_address' is not a valid IP address"
+                "The specified 'mgmt_address' is not a valid IP address."
             )
 
     @property
     def mgmt_tuple(self):
-        result = None
         Destination = namedtuple('Destination', ['ip', 'subnet'])
         try:
             parts = self._values['mgmt_address'].split('/')
@@ -367,13 +377,50 @@ class Parameters(AnsibleF5Parameters):
         if self.initial_image_exists(self._values['initial_image']):
             return self._values['initial_image']
         raise F5ModuleError(
-            "The specified 'initial_image' does not exist on the remote device"
+            "The specified 'initial_image' does not exist on the remote device."
+        )
+
+    @property
+    def initial_hotfix(self):
+        if self._values['initial_hotfix'] is None:
+            return None
+        if self.initial_hotfix_exists(self._values['initial_hotfix']):
+            return self._values['initial_hotfix']
+        raise F5ModuleError(
+            "The specified 'initial_hotfix' does not exist on the remote device."
         )
 
     def initial_image_exists(self, image):
-        collection = self.client.api.tm.sys.software.images.get_collection()
-        for resource in collection:
-            if resource.name.startswith(image):
+        uri = "https://{0}:{1}/mgmt/tm/sys/software/image/".format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
+        )
+        resp = self.client.api.get(uri)
+        try:
+            response = resp.json()
+        except ValueError:
+            return False
+        if resp.status == 404 or 'code' in response and response['code'] == 404:
+            return False
+        for resource in response['items']:
+            if resource['name'].startswith(image):
+                return True
+        return False
+
+    def initial_hotfix_exists(self, hotfix):
+        uri = "https://{0}:{1}/mgmt/tm/sys/software/hotfix/".format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
+        )
+        resp = self.client.api.get(uri)
+        try:
+            response = resp.json()
+        except ValueError:
+            return False
+        if resp.status == 404 or 'code' in response and response['code'] == 404:
+            return False
+        for resource in response['items']:
+            if resource['name'].startswith(hotfix):
                 return True
         return False
 
@@ -386,23 +433,27 @@ class Parameters(AnsibleF5Parameters):
         return result
 
 
-class ApiParameters(Parameters):
-    pass
-
-
-class ModuleParameters(Parameters):
-    pass
-
-
 class Changes(Parameters):
+    def to_return(self):
+        result = {}
+        try:
+            for returnable in self.returnables:
+                change = getattr(self, returnable)
+                if isinstance(change, dict):
+                    result.update(change)
+                else:
+                    result[returnable] = change
+            result = self._filter_params(result)
+        except Exception:
+            pass
+        return result
+
+
+class UsableChanges(Changes):
     pass
 
 
-class UsableChanges(Parameters):
-    pass
-
-
-class ReportableChanges(Parameters):
+class ReportableChanges(Changes):
     pass
 
 
@@ -432,7 +483,7 @@ class Difference(object):
         want = self.want.mgmt_tuple
         if want.subnet is None:
             raise F5ModuleError(
-                "A subnet must be specified when changing the mgmt_address"
+                "A subnet must be specified when changing the mgmt_address."
             )
         if self.want.mgmt_address != self.have.mgmt_address:
             return self.want.mgmt_address
@@ -450,9 +501,10 @@ class Difference(object):
 class ModuleManager(object):
     def __init__(self, *args, **kwargs):
         self.module = kwargs.get('module', None)
-        self.client = kwargs.get('client', None)
+        self.client = F5RestClient(**self.module.params)
         self.want = ModuleParameters(client=self.client, params=self.module.params)
-        self.changes = Changes()
+        self.have = None
+        self.changes = ReportableChanges()
 
     def _set_changed_options(self):
         changed = {}
@@ -477,6 +529,14 @@ class ModuleManager(object):
             return True
         return False
 
+    def _announce_deprecations(self, result):
+        warnings = result.pop('__warnings', [])
+        for warning in warnings:
+            self.module.deprecate(
+                msg=warning['msg'],
+                version=warning['version']
+            )
+
     def should_update(self):
         result = self._update_changed_options()
         if result:
@@ -488,39 +548,28 @@ class ModuleManager(object):
         result = dict()
         state = self.want.state
 
-        try:
-            if state in ['configured', 'provisioned', 'deployed']:
-                changed = self.present()
-            elif state == "absent":
-                changed = self.absent()
-        except iControlUnexpectedHTTPError as e:
-            raise F5ModuleError(str(e))
+        if state in ['configured', 'provisioned', 'deployed']:
+            changed = self.present()
+        elif state == "absent":
+            changed = self.absent()
 
-        changes = self.changes.to_return()
+        reportable = ReportableChanges(params=self.changes.to_return())
+        changes = reportable.to_return()
         result.update(**changes)
         result.update(dict(changed=changed))
         self._announce_deprecations(result)
         return result
 
-    def _announce_deprecations(self, result):
-        warnings = result.pop('__warnings', [])
-        for warning in warnings:
-            self.module.deprecate(
-                msg=warning['msg'],
-                version=warning['version']
-            )
-
     def present(self):
         if self.exists():
             return self.update()
         else:
             return self.create()
 
-    def exists(self):
-        result = self.client.api.tm.vcmp.guests.guest.exists(
-            name=self.want.name
-        )
-        return result
+    def absent(self):
+        if self.exists():
+            return self.remove()
+        return False
 
     def update(self):
         self.have = self.read_current_from_device()
@@ -569,44 +618,115 @@ class ModuleManager(object):
             self.configure()
         return True
 
-    def create_on_device(self):
-        params = self.want.api_params()
-        self.client.api.tm.vcmp.guests.guest.create(
-            name=self.want.name,
-            **params
+    def exists(self):
+        uri = "https://{0}:{1}/mgmt/tm/vcmp/guest/{2}".format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
+            self.want.name
         )
+        resp = self.client.api.get(uri)
+        try:
+            response = resp.json()
+        except ValueError:
+            return False
+        if resp.status == 404 or 'code' in response and response['code'] == 404:
+            return False
+        return True
+
+    def create_on_device(self):
+        params = self.changes.api_params()
+        params['name'] = self.want.name
+        uri = "https://{0}:{1}/mgmt/tm/vcmp/guest/".format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
+        )
+        resp = self.client.api.post(uri, json=params)
+        try:
+            response = resp.json()
+        except ValueError as ex:
+            raise F5ModuleError(str(ex))
+
+        if 'code' in response and response['code'] in [400, 403]:
+            if 'message' in response:
+                raise F5ModuleError(response['message'])
+            else:
+                raise F5ModuleError(resp.content)
+        return response['selfLink']
 
     def update_on_device(self):
         params = self.changes.api_params()
-        resource = self.client.api.tm.vcmp.guests.guest.load(
-            name=self.want.name
+        uri = "https://{0}:{1}/mgmt/tm/vcmp/guest/{2}".format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
+            self.want.name
         )
-        resource.modify(**params)
+        resp = self.client.api.patch(uri, json=params)
+        try:
+            response = resp.json()
+        except ValueError as ex:
+            raise F5ModuleError(str(ex))
 
-    def absent(self):
-        if self.exists():
-            return self.remove()
-        return False
+        if 'code' in response and response['code'] == 400:
+            if 'message' in response:
+                raise F5ModuleError(response['message'])
+            else:
+                raise F5ModuleError(resp.content)
 
     def remove_from_device(self):
-        resource = self.client.api.tm.vcmp.guests.guest.load(
-            name=self.want.name
+        uri = "https://{0}:{1}/mgmt/tm/vcmp/guest/{2}".format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
+            self.want.name
         )
-        if resource:
-            resource.delete()
+        response = self.client.api.delete(uri)
+        if response.status == 200:
+            return True
+        raise F5ModuleError(response.content)
 
     def read_current_from_device(self):
-        resource = self.client.api.tm.vcmp.guests.guest.load(
-            name=self.want.name
+        uri = "https://{0}:{1}/mgmt/tm/vcmp/guest/{2}".format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
+            self.want.name
         )
-        result = resource.attrs
-        return ApiParameters(params=result)
+        resp = self.client.api.get(uri)
+        try:
+            response = resp.json()
+        except ValueError as ex:
+            raise F5ModuleError(str(ex))
+
+        if 'code' in response and response['code'] == 400:
+            if 'message' in response:
+                raise F5ModuleError(response['message'])
+            else:
+                raise F5ModuleError(resp.content)
+        return ApiParameters(params=response)
 
     def remove_virtual_disk(self):
         if self.virtual_disk_exists():
             return self.remove_virtual_disk_from_device()
         return False
 
+    def get_virtual_disks_on_device(self):
+        uri = "https://{0}:{1}/mgmt/tm/vcmp/virtual-disk/".format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
+        )
+        resp = self.client.api.get(uri)
+        try:
+            response = resp.json()
+        except ValueError as ex:
+            raise F5ModuleError(str(ex))
+
+        if 'code' in response and response['code'] == 400:
+            if 'message' in response:
+                raise F5ModuleError(response['message'])
+            else:
+                raise F5ModuleError(resp.content)
+
+        if 'items' in response:
+            return response
+
     def virtual_disk_exists(self):
         """Checks if a virtual disk exists for a guest
 
@@ -624,23 +744,34 @@ class ModuleManager(object):
         for the virtual-disk without the trailing slot.
 
         Returns:
-            bool: True on success. False otherwise.
+            dict
         """
-        collection = self.client.api.tm.vcmp.virtual_disks.get_collection()
-        for resource in collection:
-            check = '{0}'.format(self.have.virtual_disk)
-            if resource.name.startswith(check):
+        response = self.get_virtual_disks_on_device()
+        check = '{0}'.format(self.have.virtual_disk)
+        for resource in response['items']:
+            if resource['name'].startswith(check):
                 return True
-        return False
+            else:
+                return False
 
     def remove_virtual_disk_from_device(self):
-        collection = self.client.api.tm.vcmp.virtual_disks.get_collection()
-        for resource in collection:
-            check = '{0}'.format(self.have.virtual_disk)
-            if resource.name.startswith(check):
-                resource.delete()
-                return True
-        return False
+        check = '{0}'.format(self.have.virtual_disk)
+        response = self.get_virtual_disks_on_device()
+        for resource in response['items']:
+            if resource['name'].startswith(check):
+                uri = "https://{0}:{1}/mgmt/tm/vcmp/virtual-disk/{2}".format(
+                    self.client.provider['server'],
+                    self.client.provider['server_port'],
+                    resource['name'].replace('/', '~')
+                )
+                response = self.client.api.delete(uri)
+
+                if response.status == 200:
+                    continue
+                else:
+                    raise F5ModuleError(response.content)
+
+        return True
 
     def is_configured(self):
         """Checks to see if guest is disabled
@@ -650,35 +781,85 @@ class ModuleManager(object):
 
         :return:
         """
+        uri = "https://{0}:{1}/mgmt/tm/vcmp/guest/{2}/stats".format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
+            self.want.name
+        )
+        resp = self.client.api.get(uri)
+
         try:
-            res = self.client.api.tm.vcmp.guests.guest.load(name=self.want.name)
-            Stats(res.stats.load())
-            return False
-        except iControlUnexpectedHTTPError as ex:
-            if 'Object not found - ' in str(ex):
-                return True
-            raise
+            response = resp.json()
+        except ValueError as ex:
+            raise F5ModuleError(str(ex))
+
+        if resp.status == 404 or 'code' in response and response['code'] == 404:
+            return True
+
+        if 'code' in response and response['code'] == 400:
+            if 'message' in response:
+                raise F5ModuleError(response['message'])
+            else:
+                raise F5ModuleError(resp.content)
+
+        return False
 
     def is_provisioned(self):
+        uri = "https://{0}:{1}/mgmt/tm/vcmp/guest/{2}/stats".format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
+            self.want.name
+        )
+        resp = self.client.api.get(uri)
+
         try:
-            res = self.client.api.tm.vcmp.guests.guest.load(name=self.want.name)
-            stats = Stats(res.stats.load())
-            if stats.stat['requestedState']['description'] == 'provisioned':
-                if stats.stat['vmStatus']['description'] == 'stopped':
+            response = resp.json()
+        except ValueError:
+            return False
+        if 'code' in response and response['code'] == 400:
+            if 'message' in response:
+                raise F5ModuleError(response['message'])
+            else:
+                raise F5ModuleError(resp.content)
+
+        if resp.status == 404 or 'code' in response and response['code'] == 404:
+            return False
+
+        result = parseStats(response)
+
+        if 'stats' in result:
+            if result['stats']['requestedState'] == 'provisioned':
+                if result['stats']['vmStatus'] == 'stopped':
                     return True
-        except iControlUnexpectedHTTPError:
-            pass
         return False
 
     def is_deployed(self):
+        uri = "https://{0}:{1}/mgmt/tm/vcmp/guest/{2}/stats".format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
+            self.want.name
+        )
+        resp = self.client.api.get(uri)
+
         try:
-            res = self.client.api.tm.vcmp.guests.guest.load(name=self.want.name)
-            stats = Stats(res.stats.load())
-            if stats.stat['requestedState']['description'] == 'deployed':
-                if stats.stat['vmStatus']['description'] == 'running':
+            response = resp.json()
+        except ValueError:
+            return False
+        if 'code' in response and response['code'] == 400:
+            if 'message' in response:
+                raise F5ModuleError(response['message'])
+            else:
+                raise F5ModuleError(resp.content)
+
+        if resp.status == 404 or 'code' in response and response['code'] == 404:
+            return False
+
+        result = parseStats(response)
+
+        if 'stats' in result:
+            if result['stats']['requestedState'] == 'deployed':
+                if result['stats']['vmStatus'] == 'running':
                     return True
-        except iControlUnexpectedHTTPError:
-            pass
         return False
 
     def configure(self):
@@ -689,8 +870,23 @@ class ModuleManager(object):
         return True
 
     def configure_on_device(self):
-        resource = self.client.api.tm.vcmp.guests.guest.load(name=self.want.name)
-        resource.modify(state='configured')
+        params = dict(state='configured')
+        uri = "https://{0}:{1}/mgmt/tm/vcmp/guest/{2}".format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
+            self.want.name
+        )
+        resp = self.client.api.patch(uri, json=params)
+        try:
+            response = resp.json()
+        except ValueError as ex:
+            raise F5ModuleError(str(ex))
+
+        if 'code' in response and response['code'] == 400:
+            if 'message' in response:
+                raise F5ModuleError(response['message'])
+            else:
+                raise F5ModuleError(resp.content)
 
     def wait_for_configured(self):
         nops = 0
@@ -706,8 +902,23 @@ class ModuleManager(object):
         self.wait_for_provisioned()
 
     def provision_on_device(self):
-        resource = self.client.api.tm.vcmp.guests.guest.load(name=self.want.name)
-        resource.modify(state='provisioned')
+        params = dict(state='provisioned')
+        uri = "https://{0}:{1}/mgmt/tm/vcmp/guest/{2}".format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
+            self.want.name
+        )
+        resp = self.client.api.patch(uri, json=params)
+        try:
+            response = resp.json()
+        except ValueError as ex:
+            raise F5ModuleError(str(ex))
+
+        if 'code' in response and response['code'] == 400:
+            if 'message' in response:
+                raise F5ModuleError(response['message'])
+            else:
+                raise F5ModuleError(resp.content)
 
     def wait_for_provisioned(self):
         nops = 0
@@ -723,8 +934,23 @@ class ModuleManager(object):
         self.wait_for_deployed()
 
     def deploy_on_device(self):
-        resource = self.client.api.tm.vcmp.guests.guest.load(name=self.want.name)
-        resource.modify(state='deployed')
+        params = dict(state='deployed')
+        uri = "https://{0}:{1}/mgmt/tm/vcmp/guest/{2}".format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
+            self.want.name
+        )
+        resp = self.client.api.patch(uri, json=params)
+        try:
+            response = resp.json()
+        except ValueError as ex:
+            raise F5ModuleError(str(ex))
+
+        if 'code' in response and response['code'] == 400:
+            if 'message' in response:
+                raise F5ModuleError(response['message'])
+            else:
+                raise F5ModuleError(resp.content)
 
     def wait_for_deployed(self):
         nops = 0
@@ -744,6 +970,7 @@ class ArgumentSpec(object):
             mgmt_address=dict(),
             mgmt_route=dict(),
             initial_image=dict(),
+            initial_hotfix=dict(),
             state=dict(
                 default='present',
                 choices=['configured', 'disabled', 'provisioned', 'absent', 'present']
@@ -774,19 +1001,15 @@ def main():
 
     module = AnsibleModule(
         argument_spec=spec.argument_spec,
-        supports_check_mode=spec.supports_check_mode
+        supports_check_mode=spec.supports_check_mode,
+        required_if=spec.required_if
     )
-    if not HAS_F5SDK:
-        module.fail_json(msg="The python f5-sdk module is required")
 
     try:
-        client = F5Client(**module.params)
-        mm = ModuleManager(module=module, client=client)
+        mm = ModuleManager(module=module)
         results = mm.exec_module()
-        cleanup_tokens(client)
         module.exit_json(**results)
     except F5ModuleError as ex:
-        cleanup_tokens(client)
         module.fail_json(msg=str(ex))
 
 
