@@ -41,6 +41,8 @@ options:
       - You can also pass a url or a local path to a rpm file (using state=present).
         To operate on several packages this can accept a comma separated string of packages or (as of 2.0) a list of packages.
     aliases: [ pkg ]
+    type: list
+    elements: str
   exclude:
     description:
       - Package name(s) to exclude when state=present, or latest
@@ -393,11 +395,11 @@ class YumModule(YumDnf):
         self.lockfile = '/var/run/yum.pid'
         self._yum_base = None
 
-    def _enablerepos_with_error_checking(self, yumbase):
+    def _enablerepos_with_error_checking(self):
         # NOTE: This seems unintuitive, but it mirrors yum's CLI behavior
         if len(self.enablerepo) == 1:
             try:
-                yumbase.repos.enableRepo(self.enablerepo[0])
+                self.yum_base.repos.enableRepo(self.enablerepo[0])
             except yum.Errors.YumBaseError as e:
                 if u'repository not found' in to_text(e):
                     self.module.fail_json(msg="Repository %s not found." % self.enablerepo[0])
@@ -406,7 +408,7 @@ class YumModule(YumDnf):
         else:
             for rid in self.enablerepo:
                 try:
-                    yumbase.repos.enableRepo(rid)
+                    self.yum_base.repos.enableRepo(rid)
                 except yum.Errors.YumBaseError as e:
                     if u'repository not found' in to_text(e):
                         self.module.warn("Repository %s not found." % rid)
@@ -492,10 +494,11 @@ class YumModule(YumDnf):
             self.yum_base.conf
 
             try:
-                self._enablerepos_with_error_checking(self._yum_base)
-
                 for rid in self.disablerepo:
                     self.yum_base.repos.disableRepo(rid)
+
+                self._enablerepos_with_error_checking()
+
             except Exception as e:
                 self.module.fail_json(msg="Failure talking to yum: %s" % to_native(e))
 
