@@ -29,6 +29,12 @@ options:
         description:
             - Name of the host
         required: true
+    use:
+        description:
+            - Which strategy to use to update the hostname.
+            - If not set we try to autodetect, but this can be problematic, specially with containers as they can present misleading information.
+        choices: ['generic', 'debian','sles', 'redhat', 'alpine', 'systemd', 'openrc', 'openbsd', 'solaris', 'freebsd']
+        version_added: '2.9'
 '''
 
 EXAMPLES = '''
@@ -50,6 +56,9 @@ from ansible.module_utils.basic import (
 from ansible.module_utils.facts.system.service_mgr import ServiceMgrFactCollector
 from ansible.module_utils._text import to_native
 
+STRATS = {'generic': 'Generic', 'debian': 'Debian', 'sles': 'SLES', 'redhat': 'RedHat', 'alpine': 'Alpine',
+          'systemd': 'Systemd', 'openrc': 'OpenRC', 'openbsd': 'OpenBSD', 'solaris': 'Solaris', 'freebsd': 'FreeBSD'}
+
 
 class UnimplementedStrategy(object):
     def __init__(self, module):
@@ -107,7 +116,12 @@ class Hostname(object):
     def __init__(self, module):
         self.module = module
         self.name = module.params['name']
-        if self.platform == 'Linux' and ServiceMgrFactCollector.is_systemd_managed(module):
+        self.use = module.params['use']
+
+        if self.use is not None:
+            strat = globals()['%sStrategy' % STRATS[self.use]]
+            self.strategy = strat(module)
+        elif self.platform == 'Linux' and ServiceMgrFactCollector.is_systemd_managed(module):
             self.strategy = SystemdStrategy(module)
         else:
             self.strategy = self.strategy_class(module)
@@ -141,8 +155,10 @@ class GenericStrategy(object):
 
     def __init__(self, module):
         self.module = module
-        self.hostname_cmd = self.module.get_bin_path('hostname', True)
         self.changed = False
+        self.hostname_cmd = self.module.get_bin_path('hostnamectl', False)
+        if not self.hostname_cmd:
+            self.hostname_cmd = self.module.get_bin_path('hostname', True)
 
     def update_current_and_permanent_hostname(self):
         self.update_current_hostname()
@@ -179,7 +195,7 @@ class GenericStrategy(object):
             self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" % (rc, out, err))
 
     def get_permanent_hostname(self):
-        return None
+        return 'UNKNOWN'
 
     def set_permanent_hostname(self, name):
         pass
@@ -360,7 +376,7 @@ class SystemdStrategy(GenericStrategy):
     """
 
     def get_current_hostname(self):
-        cmd = ['hostname']
+        cmd = [self.hostname_cmd, '--transient', 'status']
         rc, out, err = self.module.run_command(cmd)
         if rc != 0:
             self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" % (rc, out, err))
@@ -369,13 +385,13 @@ class SystemdStrategy(GenericStrategy):
     def set_current_hostname(self, name):
         if len(name) > 64:
             self.module.fail_json(msg="name cannot be longer than 64 characters on systemd servers, try a shorter name")
-        cmd = ['hostnamectl', '--transient', 'set-hostname', name]
+        cmd = [self.hostname_cmd, '--transient', 'set-hostname', name]
         rc, out, err = self.module.run_command(cmd)
         if rc != 0:
             self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" % (rc, out, err))
 
     def get_permanent_hostname(self):
-        cmd = ['hostnamectl', '--static', 'status']
+        cmd = [self.hostname_cmd, '--static', 'status']
         rc, out, err = self.module.run_command(cmd)
         if rc != 0:
             self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" % (rc, out, err))
@@ -384,11 +400,11 @@ class SystemdStrategy(GenericStrategy):
     def set_permanent_hostname(self, name):
         if len(name) > 64:
             self.module.fail_json(msg="name cannot be longer than 64 characters on systemd servers, try a shorter name")
-        cmd = ['hostnamectl', '--pretty', 'set-hostname', name]
+        cmd = [self.hostname_cmd, '--pretty', 'set-hostname', name]
         rc, out, err = self.module.run_command(cmd)
         if rc != 0:
             self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" % (rc, out, err))
-        cmd = ['hostnamectl', '--static', 'set-hostname', name]
+        cmd = [self.hostname_cmd, '--static', 'set-hostname', name]
         rc, out, err = self.module.run_command(cmd)
         if rc != 0:
             self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" % (rc, out, err))
@@ -403,20 +419,22 @@ class OpenRCStrategy(GenericStrategy):
     HOSTNAME_FILE = '/etc/conf.d/hostname'
 
     def get_permanent_hostname(self):
+        name = 'UNKNOWN'
         try:
             try:
                 f = open(self.HOSTNAME_FILE, 'r')
                 for line in f:
                     line = line.strip()
                     if line.startswith('hostname='):
-                        return line[10:].strip('"')
+                        name = line[10:].strip('"')
+                        break
             except Exception as e:
                 self.module.fail_json(msg="failed to read hostname: %s" %
                                           to_native(e), exception=traceback.format_exc())
         finally:
             f.close()
 
-        return None
+        return name
 
     def set_permanent_hostname(self, name):
         try:
@@ -515,6 +533,7 @@ class FreeBSDStrategy(GenericStrategy):
 
     def get_permanent_hostname(self):
 
+        name = 'UNKNOWN'
         if not os.path.isfile(self.HOSTNAME_FILE):
             try:
                 open(self.HOSTNAME_FILE, "a").write("hostname=temporarystub\n")
@@ -527,14 +546,15 @@ class FreeBSDStrategy(GenericStrategy):
                 for line in f:
                     line = line.strip()
                     if line.startswith('hostname='):
-                        return line[10:].strip('"')
+                        name = line[10:].strip('"')
+                        break
             except Exception as e:
                 self.module.fail_json(msg="failed to read hostname: %s" %
                                           to_native(e), exception=traceback.format_exc())
         finally:
             f.close()
 
-        return None
+        return name
 
     def set_permanent_hostname(self, name):
         try:
@@ -583,12 +603,30 @@ class OpenSUSEHostname(Hostname):
     strategy_class = SystemdStrategy
 
 
+class OpenSUSELeapHostname(Hostname):
+    platform = 'Linux'
+    distribution = 'Opensuse-leap'
+    strategy_class = SystemdStrategy
+
+
 class ArchHostname(Hostname):
     platform = 'Linux'
     distribution = 'Arch'
     strategy_class = SystemdStrategy
 
 
+class ArchARMHostname(Hostname):
+    platform = 'Linux'
+    distribution = 'Archarm'
+    strategy_class = SystemdStrategy
+
+
+class ManjaroHostname(Hostname):
+    platform = 'Linux'
+    distribution = 'Manjaro'
+    strategy_class = SystemdStrategy
+
+
 class RHELHostname(Hostname):
     platform = 'Linux'
     distribution = 'Redhat'
@@ -601,12 +639,24 @@ class CentOSHostname(Hostname):
     strategy_class = RedHatStrategy
 
 
+class ClearLinuxHostname(Hostname):
+    platform = 'Linux'
+    distribution = 'Clear-linux-os'
+    strategy_class = SystemdStrategy
+
+
 class CloudlinuxHostname(Hostname):
     platform = 'Linux'
     distribution = 'Cloudlinux'
     strategy_class = RedHatStrategy
 
 
+class CoreosHostname(Hostname):
+    platform = 'Linux'
+    distribution = 'Coreos'
+    strategy_class = SystemdStrategy
+
+
 class ScientificHostname(Hostname):
     platform = 'Linux'
     distribution = 'Scientific'
@@ -637,6 +687,12 @@ class DebianHostname(Hostname):
     strategy_class = DebianStrategy
 
 
+class CumulusHostname(Hostname):
+    platform = 'Linux'
+    distribution = 'Cumulus-linux'
+    strategy_class = DebianStrategy
+
+
 class KaliHostname(Hostname):
     platform = 'Linux'
     distribution = 'Kali'
@@ -724,7 +780,8 @@ class NeonHostname(Hostname):
 def main():
     module = AnsibleModule(
         argument_spec=dict(
-            name=dict(required=True)
+            name=dict(type='str', required=True),
+            use=dict(type='str', choices=STRATS.keys())
         ),
         supports_check_mode=True,
     )
