@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-# (c) 2012, Michael DeHaan <michael.dehaan@gmail.com>, and others
+# Copyright: (c) 2012, Michael DeHaan <michael.dehaan@gmail.com>, and others
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
 from __future__ import absolute_import, division, print_function
@@ -18,12 +18,18 @@ import traceback
 import signal
 import time
 import syslog
+import multiprocessing
+
+from ansible.module_utils._text import to_text
 
 PY3 = sys.version_info[0] == 3
 
 syslog.openlog('ansible-%s' % os.path.basename(__file__))
 syslog.syslog(syslog.LOG_NOTICE, 'Invoked with %s' % " ".join(sys.argv[1:]))
 
+# pipe for communication between forked process and parent
+ipc_watcher, ipc_notifier = multiprocessing.Pipe()
+
 
 def notice(msg):
     syslog.syslog(syslog.LOG_NOTICE, msg)
@@ -127,6 +133,11 @@ def _run_module(wrapped_cmd, jid, job_path):
     jobfile = open(tmp_job_path, "w")
     result = {}
 
+    # signal grandchild process started and isolated from being terminated
+    # by the connection being closed sending a signal to the job group
+    ipc_notifier.send(True)
+    ipc_notifier.close()
+
     outdata = ''
     filtered_outdata = ''
     stderr = ''
@@ -137,7 +148,9 @@ def _run_module(wrapped_cmd, jid, job_path):
         interpreter = _get_interpreter(cmd[0])
         if interpreter:
             cmd = interpreter + cmd
-        script = subprocess.Popen(cmd, shell=False, stdin=None, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        script = subprocess.Popen(cmd, shell=False, stdin=subprocess.PIPE, stdout=subprocess.PIPE,
+                                  stderr=subprocess.PIPE)
+
         (outdata, stderr) = script.communicate()
         if PY3:
             outdata = outdata.decode('utf-8', 'surrogateescape')
@@ -164,7 +177,7 @@ def _run_module(wrapped_cmd, jid, job_path):
         result = {
             "failed": 1,
             "cmd": wrapped_cmd,
-            "msg": str(e),
+            "msg": to_text(e),
             "outdata": outdata,  # temporary notice only
             "stderr": stderr
         }
@@ -186,8 +199,7 @@ def _run_module(wrapped_cmd, jid, job_path):
     os.rename(tmp_job_path, job_path)
 
 
-if __name__ == '__main__':
-
+def main():
     if len(sys.argv) < 5:
         print(json.dumps({
             "failed": True,
@@ -222,7 +234,7 @@ if __name__ == '__main__':
     if not os.path.exists(jobdir):
         try:
             os.makedirs(jobdir)
-        except:
+        except Exception:
             print(json.dumps({
                 "failed": 1,
                 "msg": "could not create: %s" % jobdir
@@ -239,15 +251,32 @@ if __name__ == '__main__':
             # to initialize PRIOR to ansible trying to clean up the launch directory (and argsfile)
             # this probably could be done with some IPC later.  Modules should always read
             # the argsfile at the very first start of their execution anyway
+
+            # close off notifier handle in grandparent, probably unnecessary as
+            # this process doesn't hang around long enough
+            ipc_notifier.close()
+
+            # allow waiting up to 2.5 seconds in total should be long enough for worst
+            # loaded environment in practice.
+            retries = 25
+            while retries > 0:
+                if ipc_watcher.poll(0.1):
+                    break
+                else:
+                    retries = retries - 1
+                    continue
+
             notice("Return async_wrapper task started.")
             print(json.dumps({"started": 1, "finished": 0, "ansible_job_id": jid, "results_file": job_path,
                               "_ansible_suppress_tmpdir_delete": not preserve_tmp}))
             sys.stdout.flush()
-            time.sleep(1)
             sys.exit(0)
         else:
             # The actual wrapper process
 
+            # close off the receiving end of the pipe from child process
+            ipc_watcher.close()
+
             # Daemonize, so we keep on running
             daemonize_self()
 
@@ -256,6 +285,10 @@ if __name__ == '__main__':
 
             sub_pid = os.fork()
             if sub_pid:
+                # close off inherited pipe handles
+                ipc_watcher.close()
+                ipc_notifier.close()
+
                 # the parent stops the process after the time limit
                 remaining = int(time_limit)
 
@@ -300,3 +333,7 @@ if __name__ == '__main__':
             "msg": "FATAL ERROR: %s" % e
         }))
         sys.exit(1)
+
+
+if __name__ == '__main__':
+    main()
